/***======================================================================***/
/***        UPGRADE THE CI 5.4 COMMON DATA MODEL TO VERSION 6.1           ***/
/***======================================================================***/

/***======================================================================***/
/*** NOTE: SECTION: ONE, SECTION: TWO AND SECTION: THREE REQUIRE          ***/
/***       CERTAIN MODIFICATIONS BEFORE EXECUTION OF THIS SCRIPT.         ***/
/***       PLEASE SCAN THE ENTIRE SCRIPT FOR INSTRUCTIONS PRIOR TO        ***/
/***       RUNNING THIS SCRIPT.                                           ***/
/***======================================================================***/

/***======================================================================***/
/***======================================================================***/
/***===     S E C T I O N:  O N E - M I S C E L L A N E O U S          ===***/
/***======================================================================***/
/***======================================================================***/

/***======================================================================***/
/*** BEFORE RUNNING THIS SCRIPT, WITHIN THIS SECTION SCROLL DOWN TO THE   ***/
/*** SECTIONS TITLED:                                                     ***/
/***     UPGRADE CONTACT HISTORY TABLE(S) TO 6.1                          ***/
/***     UPGRADE RESPONSE HISTORY TABLE(S) TO 6.1                         ***/
/***     UPGRADE PRESENTED TREATMENT HISTORY TABLE(S) TO 6.1              ***/
/*** MODIFY THE MACROS: %CHMigrate, %RHMigrate, %PHMigrate WITHIN THESE   ***/
/*** SECTIONS TO SUPPLY THE APPROPRIATE MACRO VARIABLES FOR YOUR CURRENT  ***/
/*** HISTORY TABLE(S).                                                    ***/
/***======================================================================***/

/*===================================================================*/
/* Enter Customer Specific Target Source Connection Values - SPDS    */
/*===================================================================*/
%let lib     =  <Library>;           /* For example, MIGRATE           */
%let domain  = '<Libname Domain>' ;  /* CI Libname Domain              */
%let server  = '<serverNode.Port>' ; /* Name Server Network Node, Port */
%let user    = '<User>';             /* SPDS User                      */
%let passwd  = '<Password>';         /* SPDS Password                  */
%let dttmfmt = <DateTime Format>;    /* For example, NLDATM21.         */
%let dtfmt   = <Date Format>;        /* For example, DATE9.            */

LIBNAME &lib SASSPDS &domain SERVER=&server USER=&user PASSWD=&passwd ;

PROC SQL NOERRORSTOP;
CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/***********************************************************/	
/*** CREATE NEW 6.1 TABLES                               ***/
/***********************************************************/	

EXECUTE (CREATE TABLE CI_CTL_GRP_CELL_X_TEST_CELL (
     CONTROL_GROUP_CELL_SK NUMERIC(10)   label='Control Group Cell SK',
     MARKETING_CELL_SK    NUMERIC(10)   label='Marketing Cell SK'
)) BY SASSPDS;

EXECUTE (CREATE UNIQUE INDEX PRIMARY_KEY ON CI_CTL_GRP_CELL_X_TEST_CELL (CONTROL_GROUP_CELL_SK, MARKETING_CELL_SK)) BY SASSPDS;

/***********************************************************/
/*** ADD NEW 6.1 COLUMNS TO EXISTING TABLES AND POPULATE ***/
/***********************************************************/

/*** ADD DELETED_FLG TO CI_CELL_PACKAGE ***/
EXECUTE (ALTER TABLE CI_CELL_PACKAGE ADD DELETED_FLG CHAR(1) LABEL='Deleted Flag') BY SASSPDS;

/*** POPULATE CI_CELL_PACKAGE.DELETED_FLG WITH DEFAULT VALUE ***/
EXECUTE (UPDATE CI_CELL_PACKAGE 
           SET DELETED_FLG = 'N') BY SASSPDS;  
  
/*** ADD RESPONSE_TRACKING_CD TO CI_CELL_PACKAGE ***/
EXECUTE (ALTER TABLE CI_CELL_PACKAGE ADD RESPONSE_TRACKING_CD VARCHAR(14) LABEL='Response Tracking Code') BY SASSPDS;
    
/*** ADD DELETED_FLG TO CI_CELL_PACKAGE_TREATMENT_CNT ***/
EXECUTE (ALTER TABLE CI_CELL_PACKAGE_TREATMENT_CNT ADD DELETED_FLG CHAR(1) LABEL='Deleted Flag') BY SASSPDS;

/*** POPULATE CI_CELL_PACKAGE_TREATMENT_CNT.DELETED_FLG WITH DEFAULT VALUE ***/
EXECUTE (UPDATE CI_CELL_PACKAGE_TREATMENT_CNT 
           SET DELETED_FLG = 'N') BY SASSPDS; 
  
/*** ADD DELETED_FLG TO CI_TREAT_CAMP_SET_X_CAMPAIGN ***/
EXECUTE (ALTER TABLE CI_TREAT_CAMP_SET_X_CAMPAIGN ADD DELETED_FLG CHAR(1) LABEL='Deleted Flag') BY SASSPDS;

/*** ADD SEQUENCE_NO TO CI_DYNAMIC_TREATMENT_ATTRIBUTE ***/
EXECUTE (ALTER TABLE CI_DYNAMIC_TREATMENT_ATTRIBUTE ADD SEQUENCE_NO NUMERIC(6) LABEL='Sequence Number') BY SASSPDS;

/*** POPULATE CI_DYNAMIC_TREATMENT_ATTRIBUTE.SEQUENCE_NO WITH DEFAULT VALUE ***/
EXECUTE (UPDATE CI_DYNAMIC_TREATMENT_ATTRIBUTE 
           SET SEQUENCE_NO = 0) BY SASSPDS;  

DISCONNECT FROM SASSPDS;
QUIT;
  
/***====================================================***/
/***====================================================***/
/***====================================================***/
/***    UPGRADE CONTACT HISTORY TABLE(S) TO 6.1         ***/
/***====================================================***/
/***====================================================***/
/***====================================================***/

%macro CHMigrate(CHTableName, ENGINE=SASSPDS);

PROC SQL NOERRORSTOP;
CONNECT TO &ENGINE (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/*** ADD OPTIMIZATION_BACKFILL_FLG TO CI_CONTACT_HISTORY ***/
EXECUTE (ALTER TABLE &CHTableName ADD OPTIMIZATION_BACKFILL_FLG CHAR(1) LABEL='Optimization Backfill Flag') BY &ENGINE;

/*** POPULATE CI_CONTACT_HISTORY.OPTIMIZATION_BACKFILL_FLG WITH DEFAULT VALUE ***/
EXECUTE (UPDATE &CHTableName 
           SET OPTIMIZATION_BACKFILL_FLG = 'N') BY &ENGINE;  
  
/*** ADD RESPONSE_TRACKING_CD TO CI_CONTACT_HISTORY ***/
EXECUTE (ALTER TABLE &CHTableName ADD RESPONSE_TRACKING_CD VARCHAR(14) LABEL='Response Tracking Code') BY &ENGINE;

DISCONNECT FROM &ENGINE; 
QUIT;

%mend;
/******************************************************************************
 | You must modify the macro call below to specify:
 |     o Contact History Table Name
 |
 | If you have multiple contact history tables, add the appropriate macro call
 | for each table.
 |    
 ******************************************************************************/
%CHMigrate(CI_CONTACT_HISTORY);
  
/***====================================================***/
/***====================================================***/
/***====================================================***/
/***    UPGRADE RESPONSE HISTORY TABLE(S) TO 6.1        ***/
/***====================================================***/
/***====================================================***/
/***====================================================***/

%macro RHMigrate(RHTableName, ENGINE=SASSPDS);

LIBNAME &lib SASSPDS &domain SERVER=&server USER=&user PASSWD=&passwd ;

/***  ADD RESPONSE_DT TO RESPONSE HISTORY TABLE ***/
PROC SQL NOERRORSTOP;

ALTER TABLE &LIB..&RHTableName ADD RESPONSE_DT DATE LABEL='Response Date';

QUIT;

PROC SQL NOERRORSTOP;
CONNECT TO &ENGINE (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/*** ADD RESPONSE_TRACKING_CD TO CI_RESPONSE_HISTORY ***/
EXECUTE (ALTER TABLE &RHTableName ADD RESPONSE_TRACKING_CD VARCHAR(14) LABEL='Response Tracking Code') BY &ENGINE;

/***  POPULATE RESPONSE_DT WITH DATE PORTION OF RESPONSE_DTTM ***/
EXECUTE (UPDATE &RHTableName
           SET RESPONSE_DT = DATEPART(RESPONSE_DTTM)) BY &ENGINE;

DISCONNECT FROM &ENGINE;
QUIT;

%mend;
/******************************************************************************
 | You must modify the macro call below to specify:
 |     o Response History Table Name
 |
 | If you have multiple response history tables, add the appropriate macro call
 | for each table.
 |    
 ******************************************************************************/
%RHMigrate(CI_RESPONSE_HISTORY);
  
/***====================================================***/
/***====================================================***/
/***====================================================***/
/*** UPGRADE PRESENTED TREATMENT HISTORY TABLE(S) TO 6.1***/
/***====================================================***/
/***====================================================***/
/***====================================================***/

%macro PHMigrate(PHTableName, ENGINE=SASSPDS);

PROC SQL NOERRORSTOP;
CONNECT TO &ENGINE (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/***  ADD PRESENTED_TREATMENT_DT TO PRESENTED TREATMENT HISTORY TABLE ***/
EXECUTE (ALTER TABLE &PHTableName ADD PRESENTED_TREATMENT_DT DATE LABEL='Presented Treatment Date') BY &ENGINE;

/*** ADD RESPONSE_TRACKING_CD TO CI_PRESENTED_TREATMENT_HISTORY ***/
EXECUTE (ALTER TABLE &PHTableName ADD RESPONSE_TRACKING_CD VARCHAR(14) LABEL='Response Tracking Code') BY &ENGINE;

/***  POPULATE PRESENTED_TREATMENT_DT WITH DATE PORTION OF PRESENTED_TREATMENT_HIST_DTTM ***/
EXECUTE (UPDATE &PHTableName
           SET PRESENTED_TREATMENT_DT = DATEPART(PRESENTED_TREATMENT_HIST_DTTM)) BY &ENGINE;

DISCONNECT FROM &ENGINE;  
QUIT;


%mend;
/******************************************************************************
 | You must modify the macro call below to specify:
 |     o Presented Treatment History Table Name
 |
 | If you have multiple presented treatment history tables, add the appropriate macro call
 | for each table.
 |    
 ******************************************************************************/
%PHMigrate(CI_PRESENTED_TREATMENT_HISTORY);

/***======================================================================***/
/***======================================================================***/
/***===         S E C T I O N:  T W O - R E S P O N S E   S K          ===***/
/***======================================================================***/
/***======================================================================***/

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*                                                                  */
/*  Note:  This code migrates the Customer Intelligence Common      */
/*         Data Model tables for US2323 (Response SK changes).      */
/*                                                                  */
/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*                                                                  */
/*  Prior to executing this code, it is REQUIRED to make the        */
/*  following updates:                                              */
/*                                                                  */
/*  1.  Execute migration of campaigns first with correct option    */
/*      to generate response SK map file (XML) to be used here.     */
/*                                                                  */
/*  2.  Modify the section (XML RESPONSE SK MAP) and specify the    */
/*      path file name for the XML with SKs obtained from campaign  */
/*      migration mentioned in step 1 above.                        */
/*                                                                  */
/*  3.  Customize the Response History Tables.                      */
/*      Scroll down to all blocks marked RESPONSE HISTORY SECTION   */
/*      and follow the instructions for modification of the         */
/*      Response History tables so that custom tables are updated.  */
/*                                                                  */
/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/

/*==================================================================*/
/*******************  XML RESPONSE SK MAP (XML) *********************/
/*==================================================================*/

/**=====================================================**/
/** Specify XML file that contains response SK map here **/
/** - This file is created during campaign migration    **/
/**   is required to map response code to SK            **/
/**=====================================================**/
%let xmlFileName = '<Full XML file path>';

%macro checkXMLFile;
	%if %sysfunc(fileexist(&xmlFileName)) %then %do;
		%put Found the file &xmlFileName ;
	%end;
	%else %do;
		%put The required file &xmlFileName does not exist ;
	%end;
%mend;

%checkXMLFile;

LIBNAME xmlfile xml &xmlFileName;
data &LIB..RESPONSE_MAP;
    set xmlfile.RESPONSE_ITEM;
run;

/** DEBUG - verify values **/
/** Columns expected:     **/
/**    RESPONSE_SK        **/
/**    RESPONSE_CD        **/
/**    RESPONSE_NM        **/
**title 'Response SK and Code Mapping';
**proc print data=&LIB..RESPONSE_MAP noobs;
**run;



/********************************************************************/
/*********************  UPDATE MIGRATION TABLES  ********************/
/********************************************************************/

/** CREATE TEMPORARY CI_RESPONSE TABLE BASED ON CODES IN CDM AND MAP **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Create a new temporary SCI_RESPONSE table **/
EXECUTE (CREATE TABLE SCI_RESPONSE AS (SELECT DISTINCT SKC.RESPONSE_SK, AA.RESPONSE_CD, AA.RESPONSE_NM
    FROM CI_RESPONSE AA, RESPONSE_MAP SKC
    WHERE AA.RESPONSE_CD = SKC.RESPONSE_CD)) BY SASSPDS;

/** This query will show codes for responses that are not published **/
**SELECT * FROM &LIB..RESPONSE_MAP WHERE RESPONSE_CD NOT IN (SELECT DISTINCT RESPONSE_CD FROM &LIB..SCI_RESPONSE);

EXECUTE (CREATE TABLE SCI_RESPONSE_X_CELL_PACKAGE AS (SELECT SRES.RESPONSE_SK,
                                                          A.RESPONSE_CD,
                                                          A.CELL_PACKAGE_SK,
                                                          A.RESPONSE_TYPE_CD,
                                                          A.ESTIMATED_RESPONSE_RT
                                                     FROM CI_RESPONSE_X_CELL_PACKAGE A, SCI_RESPONSE SRES
                                                    WHERE A.RESPONSE_CD = SRES.RESPONSE_CD)) BY SASSPDS;

EXECUTE (CREATE TABLE SCI_RESPONSE_CHANNEL_RESPONSE AS (SELECT A.RESPONSE_CHANNEL_CD, A.RESPONSE_CHANNEL_RESPONSE_CD, SRES.RESPONSE_SK
                                                       FROM CI_RESPONSE_CHANNEL_RESPONSE A, SCI_RESPONSE SRES
                                                      WHERE A.RESPONSE_CD = SRES.RESPONSE_CD)) BY SASSPDS;

/**=====================================================**/
/**              RESPONSE HISTORY SECTION               **/
/**=====================================================**/
/*  Modify this section to specify all response history  */
/*  custom tables that are being used in CDM.            */
/*  - This section creates migration tables and each     */
/*    reponse history table must be included.            */
/*                                                       */
/**=====================================================**/
EXECUTE (CREATE TABLE SCI_RESPONSE_HISTORY AS (SELECT A.CELL_PACKAGE_SK,
                                                   A.TREATMENT_SK,
                                                   A.TREATMENT_HASH_VAL,
                                                   SRES.RESPONSE_SK,
                                                   A.RESPONSE_CD,
                                                   A.SUBJECT_ID,
                                                   A.RESPONSE_DTTM,
                                                   A.RESPONSE_DT,
                                                   A.RESPONSE_CHANNEL_CD,
                                                   A.INFERRED_RESPONSE_FLG,
                                                   A.RESPONSE_VALUE_AMT,
                                                   A.EXTERNAL_RESPONSE_INFO_ID1,
                                                   A.EXTERNAL_RESPONSE_INFO_ID2,
                                                   A.RESPONSE_TRACKING_CD,
                                                   A.PROCESSED_DTTM
                                              FROM CI_RESPONSE_HISTORY A, SCI_RESPONSE SRES
                                             WHERE A.RESPONSE_CD = SRES.RESPONSE_CD)) BY SASSPDS;
/**=====================================================**/


DISCONNECT FROM SASSPDS;

QUIT;


PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Data in these tables should not exist if tables are in sync. This is **/
/** an effort to separate the inconsistent data just incase still useful **/
EXECUTE (CREATE TABLE DPCI_RESPONSE AS (SELECT RESPONSE_CD, RESPONSE_NM FROM CI_RESPONSE
                                     WHERE RESPONSE_CD NOT IN (SELECT DISTINCT RESPONSE_CD FROM SCI_RESPONSE))) BY SASSPDS;

EXECUTE (CREATE TABLE DPCI_RESPONSE_CHANNEL_RESPONSE AS (SELECT RESPONSE_CD, RESPONSE_CHANNEL_CD, RESPONSE_CHANNEL_RESPONSE_CD FROM CI_RESPONSE_CHANNEL_RESPONSE
                                                      WHERE (RESPONSE_CHANNEL_CD NOT IN (SELECT DISTINCT RESPONSE_CHANNEL_CD FROM SCI_RESPONSE_CHANNEL_RESPONSE)
                                                        AND RESPONSE_CHANNEL_RESPONSE_CD NOT IN (SELECT DISTINCT RESPONSE_CHANNEL_RESPONSE_CD FROM SCI_RESPONSE_CHANNEL_RESPONSE)))) BY SASSPDS;

EXECUTE (CREATE TABLE DPCI_RESPONSE_X_CELL_PACKAGE AS (SELECT * FROM CI_RESPONSE_X_CELL_PACKAGE
                                                        WHERE RESPONSE_CD NOT IN (SELECT DISTINCT RESPONSE_CD FROM SCI_RESPONSE_X_CELL_PACKAGE))) BY SASSPDS;

/**=====================================================**/
/**              RESPONSE HISTORY SECTION               **/
/**=====================================================**/
/*  Modify this section to specify all response history  */
/*  custom tables that are being used in CDM.            */
/*  - This section deletes missing or invalid SKs        */
/*                                                       */
/**=====================================================**/
EXECUTE (CREATE TABLE DPCI_RESPONSE_HISTORY AS (SELECT * FROM CI_RESPONSE_HISTORY
                                                 WHERE RESPONSE_CD NOT IN (SELECT DISTINCT RESPONSE_CD FROM SCI_RESPONSE_HISTORY))) BY SASSPDS;
/**=====================================================**/


DISCONNECT FROM SASSPDS;

QUIT;


/** DELETE INCONSISTENT VALUES **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** This step is performed to keep data consistent **/
EXECUTE (DELETE FROM SCI_RESPONSE WHERE (RESPONSE_SK IS NULL OR RESPONSE_SK < 0)) BY SASSPDS;

EXECUTE (DELETE FROM SCI_RESPONSE_CHANNEL_RESPONSE WHERE (RESPONSE_SK IS NULL OR RESPONSE_SK < 0)) BY SASSPDS;

EXECUTE (DELETE FROM SCI_RESPONSE_X_CELL_PACKAGE WHERE (RESPONSE_SK IS NULL OR RESPONSE_SK < 0)) BY SASSPDS;

/**=====================================================**/
/**              RESPONSE HISTORY SECTION               **/
/**=====================================================**/
/*  Modify this section to specify all response history  */
/*  custom tables that are being used in CDM.            */
/*  - This section deletes missing or invalid SKs        */
/*                                                       */
/**=====================================================**/
EXECUTE (DELETE FROM SCI_RESPONSE_HISTORY WHERE (RESPONSE_SK IS NULL OR RESPONSE_SK < 0)) BY SASSPDS;
/**=====================================================**/


DISCONNECT FROM SASSPDS;

QUIT;


/********************************************************************/
/*                                                                  */
/* Migration is complete on migration response tables but we need   */
/* to replace existing tables.                                      */
/*                                                                  */
/********************************************************************/


/********************************************************************/
/********************  REPLACE EXISTING TABLES  *********************/
/********************************************************************/

/** RENAME RESPONSE TABLES **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Update existing tables from migration tables **/
EXECUTE (DROP TABLE CI_RESPONSE) BY SASSPDS;
EXECUTE (CREATE TABLE CI_RESPONSE AS (SELECT RESPONSE_SK, RESPONSE_CD, RESPONSE_NM FROM SCI_RESPONSE)) BY SASSPDS;

EXECUTE (DROP TABLE CI_RESPONSE_X_CELL_PACKAGE) BY SASSPDS;
EXECUTE (CREATE TABLE CI_RESPONSE_X_CELL_PACKAGE AS (SELECT RESPONSE_SK,
                                                            CELL_PACKAGE_SK,
                                                            RESPONSE_TYPE_CD,
                                                            ESTIMATED_RESPONSE_RT
                                                       FROM SCI_RESPONSE_X_CELL_PACKAGE)) BY SASSPDS;

EXECUTE (DROP TABLE CI_RESPONSE_CHANNEL_RESPONSE) BY SASSPDS;
EXECUTE (CREATE TABLE CI_RESPONSE_CHANNEL_RESPONSE AS (SELECT RESPONSE_SK,
                                                              RESPONSE_CHANNEL_CD,
                                                              RESPONSE_CHANNEL_RESPONSE_CD
                                                         FROM SCI_RESPONSE_CHANNEL_RESPONSE)) BY SASSPDS;

/**=====================================================**/
/**              RESPONSE HISTORY SECTION               **/
/**=====================================================**/
/*  Modify this section to specify all response history  */
/*  custom tables that are being used in CDM.            */
/*  - This section updates existing tables with data     */
/*    from the modified tables                           */
/*                                                       */
/**=====================================================**/
EXECUTE (DROP TABLE CI_RESPONSE_HISTORY) BY SASSPDS;
EXECUTE (CREATE TABLE CI_RESPONSE_HISTORY AS (SELECT CELL_PACKAGE_SK,
                                                       TREATMENT_SK,
                                                       TREATMENT_HASH_VAL,
                                                       RESPONSE_SK,
                                                       SUBJECT_ID,
                                                       RESPONSE_DTTM,
                                                       RESPONSE_DT,
                                                       RESPONSE_CHANNEL_CD,
                                                       INFERRED_RESPONSE_FLG,
                                                       RESPONSE_VALUE_AMT,
                                                       EXTERNAL_RESPONSE_INFO_ID1,
                                                       EXTERNAL_RESPONSE_INFO_ID2,
                                                       RESPONSE_TRACKING_CD,
                                                       PROCESSED_DTTM
                                                  FROM SCI_RESPONSE_HISTORY)) BY SASSPDS;
/**=====================================================**/


DISCONNECT FROM SASSPDS;

QUIT;


/********************************************************************/
/**********  RESTORE CONSTRAINTS ON EXISTING TABLES  ****************/
/********************************************************************/

/** ADD PK CONSTRAINTS **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Add PK constraints **/
EXECUTE(CREATE UNIQUE INDEX RESPONSE_SK ON CI_RESPONSE (RESPONSE_SK)) BY SASSPDS;
EXECUTE(CREATE UNIQUE INDEX PRIMARY_KEY ON CI_RESPONSE_X_CELL_PACKAGE (RESPONSE_SK,CELL_PACKAGE_SK)) BY SASSPDS;
EXECUTE(CREATE UNIQUE INDEX PRIMARY_KEY ON CI_RESPONSE_CHANNEL_RESPONSE (RESPONSE_CHANNEL_CD,RESPONSE_CHANNEL_RESPONSE_CD,RESPONSE_SK)) BY SASSPDS;

/**=====================================================**/
/**              RESPONSE HISTORY SECTION               **/
/**=====================================================**/
/*  Modify this section to specify all response history  */
/*  custom tables that are being used in CDM.            */
/*  - This section adds primary and foreign key          */
/*    constraints on modified tables                     */
/*                                                       */
/**=====================================================**/
EXECUTE(CREATE UNIQUE INDEX PRIMARY_KEY ON CI_RESPONSE_HISTORY (CELL_PACKAGE_SK,TREATMENT_SK,TREATMENT_HASH_VAL,RESPONSE_SK,SUBJECT_ID,RESPONSE_DTTM)) BY SASSPDS;
/**=====================================================**/


DISCONNECT FROM SASSPDS;

QUIT;


/********************************************************************/
/**************  REMOVE MIGRATION AND BACKUP TABLES  ****************/
/********************************************************************/

/** DELETE MIGRATION TABLES **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Drop migration tables **/
EXECUTE (DROP TABLE SCI_RESPONSE_CHANNEL_RESPONSE) BY SASSPDS;
EXECUTE (DROP TABLE SCI_RESPONSE) BY SASSPDS;

EXECUTE (DROP TABLE SCI_RESPONSE_X_CELL_PACKAGE) BY SASSPDS;

/**=====================================================**/
/**              RESPONSE HISTORY SECTION               **/
/**=====================================================**/
/*  Modify this section to specify all response history  */
/*  custom tables that are being used in CDM.            */
/*  - This section deletes migration and backup tables   */
/*                                                       */
/**=====================================================**/
EXECUTE (DROP TABLE SCI_RESPONSE_HISTORY) BY SASSPDS;
/**=====================================================**/


DISCONNECT FROM SASSPDS;

QUIT;


/********************************************************************/
/*****************  REMOVE TMP WORK SPACE TABLES  *******************/
/********************************************************************/

/** DELETE WORK TABLES (optional) **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/* Cleanup temp work space */
EXECUTE(DROP TABLE RESPONSE_MAP) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/*===================================================================*/
/*                        END RESPONSE_SK                            */
/*===================================================================*/

/***======================================================================***/
/***======================================================================***/
/***===     S E C T I O N:  T H R E E - C H E C K L I S T              ===***/
/***======================================================================***/
/***======================================================================***/

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*                                                                  */
/*  Note:  This code migrates the Customer Intelligence Common      */
/*         Data Model tables for US2935 (Checklist changes).        */
/*                                                                  */
/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/

/********************************************************************/
/*********************  CI CAMPAIGN TABLES  *************************/
/********************************************************************/


/** CREATE BACKUP OF MODIFIED TABLES TO DO THE WORK **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Create a backup of existing tables to be migrated (just incase) **/
EXECUTE (CREATE TABLE SV_CI_CAMP_CHECKLIST_ITEM AS (SELECT * FROM CI_CAMP_CHECKLIST_ITEM)) BY SASSPDS;
EXECUTE (CREATE TABLE SV_CI_CAMP_CHECKLIST_ITEM_EXT AS (SELECT * FROM CI_CAMP_CHECKLIST_ITEM_EXT)) BY SASSPDS;

EXECUTE (CREATE TABLE SV_CI_CAMPAIGN_EXT AS (SELECT * FROM CI_CAMPAIGN_EXT)) BY SASSPDS;

EXECUTE (CREATE TABLE SV_CI_CAMPAIGN_CHAR_UDF AS (SELECT * FROM CI_CAMPAIGN_CHAR_UDF)) BY SASSPDS;
EXECUTE (CREATE TABLE SV_CI_CAMPAIGN_NUM_UDF AS (SELECT * FROM CI_CAMPAIGN_NUM_UDF)) BY SASSPDS;
EXECUTE (CREATE TABLE SV_CI_CAMPAIGN_DATE_UDF AS (SELECT * FROM CI_CAMPAIGN_DATE_UDF)) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


%macro UpdateBriefItem(ItemTable);

data &ItemTable; set &LIB..&ItemTable;
   if CHECKLIST_NM eq 'Brief' then do;
      CHECKLIST_NM = 'Properties Custom Details';
   end;
   else do;
      CHECKLIST_NM = trim(CHECKLIST_NM) || ' Custom Details';
   end;
run;

PROC SQL NOERRORSTOP;

/** Update custom page names **/
DELETE FROM &LIB..SV_&ItemTable;

INSERT INTO &LIB..SV_&ItemTable SELECT * FROM WORK.&ItemTable;

QUIT;

%mend;


/** UPDATE CHECKLIST NAME COLUMN **/
%UpdateBriefItem(CI_CAMP_CHECKLIST_ITEM);


/** UPDATE DATA IN THE NEW UDF COLUMN **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Update column CHECKLIST_NM values in UDF tables **/
EXECUTE (CREATE TABLE SV_CI_CAMP_CHECKLIST_CHAR_UDF AS SELECT A.CAMPAIGN_SK,
                                                           B.CHECKLIST_NM,
                                                           A.UDF_SEQ_NO,
                                                           A.CHAR_UDF_NM,
                                                           A.CHAR_UDF_VAL,
                                                           A.CHAR_EXT_COLUMN_NM,
                                                           A.PROCESSED_DTTM
                                                      FROM CI_CAMP_CHECKLIST_CHAR_UDF A, SV_CI_CAMP_CHECKLIST_ITEM B
                                                     WHERE A.CAMPAIGN_SK = B.CAMPAIGN_SK AND A.UDF_SEQ_NO = B.UDF_SEQ_NO) BY SASSPDS;

EXECUTE (CREATE TABLE SV_CI_CAMP_CHECKLIST_NUM_UDF AS SELECT A.CAMPAIGN_SK,
                                                          B.CHECKLIST_NM,
                                                          A.UDF_SEQ_NO,
                                                          A.NUM_UDF_NM,
                                                          A.NUM_UDF_VAL,
                                                          A.NUM_EXT_COLUMN_NM,
                                                          A.PROCESSED_DTTM
                                                     FROM CI_CAMP_CHECKLIST_NUM_UDF A, SV_CI_CAMP_CHECKLIST_ITEM B
                                                    WHERE A.CAMPAIGN_SK = B.CAMPAIGN_SK AND A.UDF_SEQ_NO = B.UDF_SEQ_NO) BY SASSPDS;

EXECUTE (CREATE TABLE SV_CI_CAMP_CHECKLIST_DATE_UDF AS SELECT A.CAMPAIGN_SK,
                                                           B.CHECKLIST_NM,
                                                           A.UDF_SEQ_NO,
                                                           A.DATE_UDF_NM,
                                                           A.DATE_UDF_VAL,
                                                           A.DATE_EXT_COLUMN_NM,
                                                           A.PROCESSED_DTTM
                                                      FROM CI_CAMP_CHECKLIST_DATE_UDF A, SV_CI_CAMP_CHECKLIST_ITEM B
                                                     WHERE A.CAMPAIGN_SK = B.CAMPAIGN_SK AND A.UDF_SEQ_NO = B.UDF_SEQ_NO) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** POPULATE BRIEF DATA **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

EXECUTE (CREATE TABLE TM_CI_CAMP_CHECKLIST_ITEM AS (SELECT * FROM SV_CI_CAMP_CHECKLIST_ITEM)) BY SASSPDS;

/** Add Brief Page if not already present **/
EXECUTE (INSERT INTO SV_CI_CAMP_CHECKLIST_ITEM
      (CAMPAIGN_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       CHECKLIST_STATUS_FLG)
   SELECT DISTINCT CAMPAIGN_SK,
          -1,
          'Brief Custom Details',
          'Y'
   FROM SV_CI_CAMPAIGN_EXT A
   WHERE NOT EXISTS (SELECT DISTINCT CAMPAIGN_SK FROM TM_CI_CAMP_CHECKLIST_ITEM B
                     WHERE B.CAMPAIGN_SK = A.CAMPAIGN_SK
                       AND B.CHECKLIST_NM = 'Brief Custom Details')) BY SASSPDS;

EXECUTE (DELETE FROM TM_CI_CAMP_CHECKLIST_ITEM) BY SASSPDS;
EXECUTE (INSERT INTO TM_CI_CAMP_CHECKLIST_ITEM SELECT * FROM SV_CI_CAMP_CHECKLIST_ITEM) BY SASSPDS;

/** Add empty columns if not already there **/
EXECUTE (INSERT INTO SV_CI_CAMP_CHECKLIST_ITEM
      (CAMPAIGN_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       CHECKLIST_STATUS_FLG)
   SELECT DISTINCT CAMPAIGN_SK,
          -1,
          'Brief Custom Details',
          'Y'
   FROM SV_CI_CAMPAIGN_CHAR_UDF A
   WHERE NOT EXISTS (SELECT DISTINCT CAMPAIGN_SK FROM TM_CI_CAMP_CHECKLIST_ITEM B
                     WHERE B.CAMPAIGN_SK = A.CAMPAIGN_SK
                       AND B.UDF_SEQ_NO = -1)) BY SASSPDS;

EXECUTE (DELETE FROM TM_CI_CAMP_CHECKLIST_ITEM) BY SASSPDS;
EXECUTE (INSERT INTO TM_CI_CAMP_CHECKLIST_ITEM SELECT * FROM SV_CI_CAMP_CHECKLIST_ITEM) BY SASSPDS;

EXECUTE (INSERT INTO SV_CI_CAMP_CHECKLIST_ITEM
      (CAMPAIGN_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       CHECKLIST_STATUS_FLG)
   SELECT DISTINCT CAMPAIGN_SK,
          -1,
          'Brief Custom Details',
          'Y'
   FROM SV_CI_CAMPAIGN_NUM_UDF A
   WHERE NOT EXISTS (SELECT DISTINCT CAMPAIGN_SK FROM TM_CI_CAMP_CHECKLIST_ITEM B
                     WHERE B.CAMPAIGN_SK = A.CAMPAIGN_SK
                       AND B.UDF_SEQ_NO = -1)) BY SASSPDS;

EXECUTE (DELETE FROM TM_CI_CAMP_CHECKLIST_ITEM) BY SASSPDS;
EXECUTE (INSERT INTO TM_CI_CAMP_CHECKLIST_ITEM SELECT * FROM SV_CI_CAMP_CHECKLIST_ITEM) BY SASSPDS;

EXECUTE (INSERT INTO SV_CI_CAMP_CHECKLIST_ITEM
      (CAMPAIGN_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       CHECKLIST_STATUS_FLG)
   SELECT DISTINCT CAMPAIGN_SK,
          -1,
          'Brief Custom Details',
          'Y'
   FROM SV_CI_CAMPAIGN_DATE_UDF A
   WHERE NOT EXISTS (SELECT DISTINCT CAMPAIGN_SK FROM TM_CI_CAMP_CHECKLIST_ITEM B
                     WHERE B.CAMPAIGN_SK = A.CAMPAIGN_SK
                       AND B.UDF_SEQ_NO = -1)) BY SASSPDS;

EXECUTE (DELETE FROM TM_CI_CAMP_CHECKLIST_ITEM) BY SASSPDS;
EXECUTE (INSERT INTO TM_CI_CAMP_CHECKLIST_ITEM SELECT * FROM SV_CI_CAMP_CHECKLIST_ITEM) BY SASSPDS;

/** Update processed date value (not necessary - values should exist) **/
/*UPDATE &LIB..SV_CI_CAMP_CHECKLIST_ITEM A
   SET PROCESSED_DTTM = (SELECT PROCESSED_DTTM FROM &LIB..CI_CAMPAIGN B
                         WHERE (A.CAMPAIGN_SK = B.CAMPAIGN_SK AND B.PROCESSED_DTTM IS NOT NULL))
   WHERE A.CAMPAIGN_SK IN (SELECT CAMPAIGN_SK FROM &LIB..CI_CAMPAIGN) AND A.UDF_SEQ_NO = -1;*/

/** Copy Brief UDFs **/
EXECUTE (INSERT INTO SV_CI_CAMP_CHECKLIST_CHAR_UDF
      (CAMPAIGN_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       CHAR_UDF_NM,
       CHAR_UDF_VAL,
       CHAR_EXT_COLUMN_NM,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_SK,
          -1,
          'Brief Custom Details',
          CHAR_UDF_NM,
          CHAR_UDF_VAL,
          CHAR_EXT_COLUMN_NM,
          PROCESSED_DTTM
   FROM SV_CI_CAMPAIGN_CHAR_UDF) BY SASSPDS;

EXECUTE (INSERT INTO SV_CI_CAMP_CHECKLIST_NUM_UDF
      (CAMPAIGN_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       NUM_UDF_NM,
       NUM_UDF_VAL,
       NUM_EXT_COLUMN_NM,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_SK,
          -1,
          'Brief Custom Details',
          NUM_UDF_NM,
          NUM_UDF_VAL,
          NUM_EXT_COLUMN_NM,
          PROCESSED_DTTM
   FROM SV_CI_CAMPAIGN_NUM_UDF) BY SASSPDS;

EXECUTE (INSERT INTO SV_CI_CAMP_CHECKLIST_DATE_UDF
      (CAMPAIGN_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       DATE_UDF_NM,
       DATE_UDF_VAL,
       DATE_EXT_COLUMN_NM,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_SK,
          -1,
          'Brief Custom Details',
          DATE_UDF_NM,
          DATE_UDF_VAL,
          DATE_EXT_COLUMN_NM,
          PROCESSED_DTTM
   FROM SV_CI_CAMPAIGN_DATE_UDF) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** REMOVE NULL OR DUPLICATES ON PK COLUMNS **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Remove rows with null on page name (PK) **/
EXECUTE (DELETE FROM SV_CI_CAMP_CHECKLIST_CHAR_UDF WHERE CHECKLIST_NM IS NULL) BY SASSPDS;
EXECUTE (DELETE FROM SV_CI_CAMP_CHECKLIST_NUM_UDF WHERE CHECKLIST_NM IS NULL) BY SASSPDS;
EXECUTE (DELETE FROM SV_CI_CAMP_CHECKLIST_DATE_UDF WHERE CHECKLIST_NM IS NULL) BY SASSPDS;
EXECUTE (DELETE FROM SV_CI_CAMP_CHECKLIST_ITEM WHERE CHECKLIST_NM IS NULL) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/***********************************************/
/*********** Start merge EXT macros ************/

%macro extunion(EXPANDED_TABLE, TARGET_TABLE, SAVE_TABLE, pk_column);

proc sql noprint;
 create table WORK.UNION_ALL
      as select * from
	  &TARGET_TABLE t0 LEFT OUTER JOIN &EXPANDED_TABLE t1 on t1.&pk_column. = t0.&pk_column.;
quit;

/* examine the columns */
proc contents noprint data=&TARGET_TABLE out=CONT_TARGET; run;
proc contents noprint data=&EXPANDED_TABLE out=CONT_EXPANDED; run;
proc contents noprint data=WORK.UNION_ALL out=UNION_TABLE; run;
data CONT_TARGET; set CONT_TARGET; name=upcase(name); run;
data CONT_EXPANDED; set CONT_EXPANDED; name=upcase(name); run;
data UNION_TABLE; set UNION_TABLE; name=upcase(name); run;

proc sql noprint;
   create table WORK.CONT_UNION
      as select * from
	  UNION_TABLE t0 LEFT OUTER JOIN CONT_EXPANDED t1 on (t1.type = t0.type AND t1.name = t0.name)
      where t0.name <> "&pk_column.";
quit;

%let unionVars=&pk_column.;
data _null_;
   length v $ 32767;
   retain v "&pk_column.";
   set WORK.CONT_UNION end=done;
   if name ^= "UDF_SEQ_NO" then do;
   v=trim(v)||", "||name;
   end;
   if done then do;
      call symput('unionVars',trim(v));
   end;
run;
%put unionVars=&unionVars;

filename tempfile temp lrecl=32767;
data _null_;
   length c $ 32767;
   file tempfile; /* start over */
   set WORK.CONT_UNION end=done;
   if 1=_n_ then do;
      put "LIBNAME &lib SASSPDS &domain SERVER=&server USER=&user;";
      put "PROC SQL NOERRORSTOP;";
	  put "CREATE TABLE &SAVE_TABLE. AS (SELECT &unionVars. FROM WORK.UNION_ALL);";
   end;
   if done then do;
	  put "QUIT;";
      put "run;";
   end;
run;

%include tempfile;
filename tempfile clear;


PROC SQL NOERRORSTOP;

/** Clear joined table - data will be added later **/
DELETE FROM &SAVE_TABLE;

QUIT;

%mend;


%macro extmigrate(EXPANDED_TABLE, TARGET_TABLE, SAVE_TABLE, pk_column);

/* set these variables for proper execution */
/*%let EXPANDED_TABLE=&LIB..CI_CAMP_CHECKLIST_ITEM_EXT; */
/*%let TARGET_TABLE=&LIB..CI_CAMPAIGN_EXT; */

/*%let SAVE_TABLE=&LIB..MIG_CI_CAMPAIGN_EXT; */
/*%let pk_column=CAMPAIGN_SK; */


/* examine the columns */
proc contents noprint data=&TARGET_TABLE out=CONT_TARGET; run;
proc contents noprint data=&EXPANDED_TABLE out=CONT_EXPANDED; run;
data CONT_TARGET; set CONT_TARGET; name=upcase(name); run;
data CONT_EXPANDED; set CONT_EXPANDED; name=upcase(name); run;

proc sql noprint;
   create table WORK.CONT_INTERSECTION
      as select t0.* from
	  CONT_TARGET t0 INNER JOIN CONT_EXPANDED t1 on (t0.name=t1.name and t0.type=t1.type)
	  where
	     t0.name <> "&pk_column.";
quit;

%let usedColExpr=;
data _null_;
   length colExpr $ 32767;
   retain re colExpr;
   set WORK.CONT_INTERSECTION end=done;
   if 1=_n_ then do; /* first row */
      re=prxparse("/^_\d+/");
	  if missing(re) then put "Missing!";
   end;

   if prxmatch(re,trim(name)) then do;
      if missing(colExpr) then colExpr=name;
	  else colExpr=trim(colExpr)||"|"||name;
   end;

   if done then do;
      /* wrap the regular expression if not empty and output it to a variabe */
      colExpr="/^("||trim(colExpr)||")$/";
      call symput("usedColExpr",trim(colExpr));
   end;
run;
%put ex=&usedColExpr;

/* build a list of variables to keep */
%let keepVars=&pk_column.;
data _null_;
   length v $ 32767;
   retain v "&pk_column.";
   set WORK.CONT_INTERSECTION end=done;
   v=trim(v)||" "||name;
   if done then do;
      call symput('keepVars',trim(v));
   end;
run;
%put keepVars=&keepVars;

%let renameVars=;
data WORK.CONT_INTERSECTION;
   length renameVars $ 32767;
   length tmpname $ 32;
   retain i 0;
   retain re renameVars;
   drop i re renameVars;
   set WORK.CONT_INTERSECTION end=done;
   if 1 = _n_ then do;
      re=prxparse("&usedColExpr");
   end;
   i + 1;
   tmpname="_"||left(i);
   do while (prxmatch(re,trim(tmpname)));
       i + 1;
	   tmpname="_"||left(i);
   end;

   if missing(renameVars) then renameVars=trim(name)||"="||trim(tmpname);
   else renameVars=trim(renameVars)||" "||trim(name)||"="||trim(tmpname);

   if done then do;
      if ^missing(renameVars) then renameVars="rename=("||trim(renameVars)||")";
      call symput("renameVars",trim(renameVars));
   end;
run;
%put renameVars=&renameVars;


/* drag the target table back into SAS so we can work on it */
data WORK.DESTINATION_TABLE;
   set &TARGET_TABLE;
run;
proc sort data=WORK.DESTINATION_TABLE; by &pk_column.; run;

proc sql noprint;
   create table WORK.SORTED_ITEM_EXT as
   select * from &EXPANDED_TABLE(keep=&keepVars udf_seq_no) order by &pk_column., udf_seq_no;
quit;

/* create the code necessary to merge the data properly */
filename tempfile temp lrecl=32767;
data _null_;
   file tempfile;
   put "data WORK.SORTED_ITEM_EXT;";
run;

%let dropVars=;
data _null_;
   length dropcols $ 32767;
   retain dropcols;
   length c $ 32767;
   file tempfile mod;
   set WORK.CONT_INTERSECTION end=done;
   c=".";
   if (1=type) then do;
      /*numeric*/
	  if 0 ^= formatd then c=trim(c)||trim(left(formatd));
   end;
   c="format "||trim(tmpname)||" "||trim(format)||trim(left(formatl))||trim(c)||";";
   dropcols=trim(dropcols)||" "||tmpname;
   if (not missing(format) or 0 gt formatl) then do;
      /* missing format in double precision (NTZ)! */
      put c;
   end;
   if done then do;
      call symput("dropVars", trim(dropcols));
      put "retain " dropcols ";";
      put "drop UDF_SEQ_NO " dropcols ";";
	  put "set WORK.SORTED_ITEM_EXT; by &pk_column.;";
	  put "if first.&pk_column. then do;";
   end;
run;

/* initial assignment */
data _null_;
   length c $ 32767;
   file tempfile mod;
   set WORK.CONT_INTERSECTION end=done;
   c="   "||trim(tmpname)||"="||trim(name)||";";
   put c;
   if done then put "end; else do;";
run;

/* coalescing multiple rows */
data _null_;
   length c $ 32767;
   file tempfile mod;
   set WORK.CONT_INTERSECTION end=done;
   c="";
   if (2=type) then c="c"; /* call coalesce or coalescec as appropriate */
   c="   "||trim(tmpname)||"=coalesce"||trim(c)||"("||trim(tmpname)||","||trim(name)||");";
   put c;
   if done then do;
      put "end;";
	  put "if last.&pk_column. then do;";
   end;
run;

/* assigning and outputting the coalesced rows */
data _null_;
   length c $ 32767;
   file tempfile mod;
   set WORK.CONT_INTERSECTION end=done;
   c="   "||trim(name)||"="||trim(tmpname)||";";
   put c;
   if done then do;
	  put "   output;";
	  put "end;";
	  put "run;";
   end;
run;

%include tempfile;
filename tempfile clear;

%let outData=WORK.DESTINATION_TABLE;

filename tempfile temp lrecl=32767;
data _null_;
   length c $ 32767;
   file tempfile; /* start over */
   set WORK.CONT_INTERSECTION end=done;
   if 1=_n_ then do;
      put "data WORK.OUTPUT_TABLE(drop=&dropVars);";
      put "merge WORK.DESTINATION_TABLE WORK.SORTED_ITEM_EXT(&renameVars); by &pk_column.;";
      call symput("outData", "WORK.OUTPUT_TABLE");
   end;

   c="";
   if (2=type) then c="c";
   c=trim(name)||"=coalesce"||trim(c)||"("||trim(name)||","||trim(tmpname)||");";
   put c;

   if done then do;
      put "run;";
   end;
run;

%include tempfile;
filename tempfile clear;

%put outData=&outData.;

PROC SQL NOERRORSTOP;

/** Save data to be moved to targer later **/
CREATE TABLE &SAVE_TABLE AS (SELECT * FROM &outData.);

QUIT;

%mend;

/************* End merge EXT macros ************/
/***********************************************/


/*** Run macro to add all columns to EXT ***/
%extunion(&LIB..SV_CI_CAMP_CHECKLIST_ITEM_EXT, &LIB..SV_CI_CAMPAIGN_EXT, &LIB..SMIG_CI_CAMPAIGN_EXT, CAMPAIGN_SK);

PROC APPEND BASE=&LIB..SMIG_CI_CAMPAIGN_EXT DATA=&LIB..SV_CI_CAMPAIGN_EXT FORCE; run;

/*** Run macro to migrate campaign EXT ***/
%extmigrate(&LIB..SV_CI_CAMP_CHECKLIST_ITEM_EXT, &LIB..SMIG_CI_CAMPAIGN_EXT, &LIB..MIG_CI_CAMPAIGN_EXT, CAMPAIGN_SK);


/** CREATE NEW TABLES **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Create Tables **/
EXECUTE (CREATE TABLE CI_CAMP_PAGE
(
	CAMPAIGN_SK          NUMERIC(10)  label='Campaign Surrogate Key',
	PAGE_NM              VARCHAR(60) label='PAGE Name',
	PAGE_STATUS_FLG CHARACTER(1) label='PAGE Status Flag',
	PROCESSED_DTTM       DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Processed Datetime'
)) BY SASSPDS;

EXECUTE (CREATE TABLE CI_CAMP_PAGE_CHAR_UDF
(
	CAMPAIGN_SK          NUMERIC(10)  label='Campaign Surrogate Key',
	PAGE_NM              VARCHAR(60) label='Page Name',
	CHAR_UDF_NM          VARCHAR(60) label='Char Udf Name',
	CHAR_UDF_VAL         VARCHAR(500) label='Char Udf Val',
	CHAR_EXT_COLUMN_NM   VARCHAR(30) label='Char Ext Column Name',
	PROCESSED_DTTM       DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Processed Datetime'
)) BY SASSPDS;

EXECUTE (CREATE TABLE CI_CAMP_PAGE_NUM_UDF
(
	CAMPAIGN_SK          NUMERIC(10)  label='Campaign Surrogate Key',
	PAGE_NM              VARCHAR(60) label='Page Name',
	NUM_UDF_NM           VARCHAR(60) label='Num Udf Name',
	NUM_UDF_VAL          NUMERIC(14,2) label='Num Udf Val',
	NUM_EXT_COLUMN_NM    VARCHAR(30) label='Num Ext Column Name',
	PROCESSED_DTTM       DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Processed Datetime'
)) BY SASSPDS;

EXECUTE (CREATE TABLE CI_CAMP_PAGE_DATE_UDF
(
	CAMPAIGN_SK          NUMERIC(10)  label='Campaign Surrogate Key',
	PAGE_NM              VARCHAR(60) label='Page Name',
	DATE_UDF_NM          VARCHAR(60) label='Date Udf Name',
	DATE_UDF_VAL         DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Date Udf Val',
	DATE_EXT_COLUMN_NM   VARCHAR(30) label='Date Ext Column Name',
	PROCESSED_DTTM       DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Processed Datetime'
)) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** ADD CONSTRAINTS TO NEW TABLES **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Create constraints on new tables **/
EXECUTE (CREATE UNIQUE INDEX PRIMARY_KEY ON CI_CAMP_PAGE (CAMPAIGN_SK, PAGE_NM)) BY SASSPDS;

EXECUTE (CREATE UNIQUE INDEX PRIMARY_KEY ON CI_CAMP_PAGE_CHAR_UDF (CAMPAIGN_SK, PAGE_NM, CHAR_UDF_NM)) BY SASSPDS;

EXECUTE (CREATE UNIQUE INDEX PRIMARY_KEY ON CI_CAMP_PAGE_NUM_UDF (CAMPAIGN_SK, PAGE_NM, NUM_UDF_NM)) BY SASSPDS;

EXECUTE (CREATE UNIQUE INDEX PRIMARY_KEY ON CI_CAMP_PAGE_DATE_UDF (CAMPAIGN_SK, PAGE_NM, DATE_UDF_NM)) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** POPULATE NEW TABLES FROM EXISTING DATA **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Add Existing data to new tables **/
EXECUTE (INSERT INTO CI_CAMP_PAGE
      (CAMPAIGN_SK,
       PAGE_NM,
       PAGE_STATUS_FLG,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_SK,
          CHECKLIST_NM,
          CHECKLIST_STATUS_FLG,
          PROCESSED_DTTM
   FROM SV_CI_CAMP_CHECKLIST_ITEM) BY SASSPDS;

EXECUTE (INSERT INTO CI_CAMP_PAGE_CHAR_UDF
      (CAMPAIGN_SK,
       PAGE_NM,
       CHAR_UDF_NM,
       CHAR_UDF_VAL,
       CHAR_EXT_COLUMN_NM,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_SK,
          CHECKLIST_NM,
          CHAR_UDF_NM,
          CHAR_UDF_VAL,
          CHAR_EXT_COLUMN_NM,
          PROCESSED_DTTM
   FROM SV_CI_CAMP_CHECKLIST_CHAR_UDF) BY SASSPDS;

EXECUTE (INSERT INTO CI_CAMP_PAGE_NUM_UDF
      (CAMPAIGN_SK,
       PAGE_NM,
       NUM_UDF_NM,
       NUM_UDF_VAL,
       NUM_EXT_COLUMN_NM,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_SK,
          CHECKLIST_NM,
          NUM_UDF_NM,
          NUM_UDF_VAL,
          NUM_EXT_COLUMN_NM,
          PROCESSED_DTTM
   FROM SV_CI_CAMP_CHECKLIST_NUM_UDF) BY SASSPDS;

EXECUTE (INSERT INTO CI_CAMP_PAGE_DATE_UDF
      (CAMPAIGN_SK,
       PAGE_NM,
       DATE_UDF_NM,
       DATE_UDF_VAL,
       DATE_EXT_COLUMN_NM,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_SK,
          CHECKLIST_NM,
          DATE_UDF_NM,
          DATE_UDF_VAL,
          DATE_EXT_COLUMN_NM,
          PROCESSED_DTTM
   FROM SV_CI_CAMP_CHECKLIST_DATE_UDF) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** Macro to update duplicate ext column names **/
%macro UpdateColumns(udfTable, skCol, udfNmCol, udfValCol, extCol, pageNm=PAGE_NM, procDttm=PROCESSED_DTTM);

/* Locate Brief Columns */
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

EXECUTE (CREATE TABLE X&udfTable AS (SELECT &skCol, &pageNm, &udfNmCol, &udfValCol, &extCol, &procDttm FROM &udfTable)) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;

/* Update column  name case */
PROC SQL NOERRORSTOP;

UPDATE &LIB..X&udfTable SET &extCol = upper(&extCol);

QUIT;


/* Locate Duplicate columns */
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

EXECUTE (CREATE TABLE BX&udfTable AS (SELECT &skCol, &pageNm, &udfNmCol, &extCol AS &extCol FROM X&udfTable
                                    WHERE &extCol IS NOT NULL AND &pageNm = 'Brief Custom Details')) BY SASSPDS;

EXECUTE (CREATE TABLE DX&udfTable AS SELECT &skCol, &pageNm, &udfNmCol, &extCol AS &extCol, count(&extCol) AS cnt FROM X&udfTable
                                   WHERE &extCol IS NOT NULL AND NOT &pageNm = 'Brief Custom Details'
                                GROUP BY &skCol, &extCol HAVING cnt > 1) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/* Update Existing UDF Tables */
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

EXECUTE (CREATE TABLE BA&udfTable AS (SELECT * FROM X&udfTable A
         WHERE EXISTS (SELECT A.&skCol, A.&extCol FROM BX&udfTable B WHERE A.&skCol = B.&skCol AND A.&extCol = B.&extCol)
           AND NOT &pageNm = 'Brief Custom Details')) BY SASSPDS;

EXECUTE (CREATE TABLE DA&udfTable AS (SELECT * FROM X&udfTable A
         WHERE EXISTS (SELECT A.&skCol, A.&extCol FROM DX&udfTable B WHERE A.&skCol = B.&skCol AND A.&extCol = B.&extCol)
           AND NOT &pageNm = 'Brief Custom Details')) BY SASSPDS;

EXECUTE (UPDATE BA&udfTable SET &extCol = '') BY SASSPDS;

EXECUTE (UPDATE DA&udfTable SET &extCol = '') BY SASSPDS;

EXECUTE (DELETE FROM X&udfTable A
         WHERE EXISTS (SELECT A.&skCol, A.&extCol FROM BX&udfTable B WHERE A.&skCol = B.&skCol AND A.&extCol = B.&extCol)
           AND NOT &pageNm = 'Brief Custom Details') BY SASSPDS;

EXECUTE (DELETE FROM X&udfTable A
         WHERE EXISTS (SELECT A.&skCol, A.&extCol FROM DX&udfTable B WHERE A.&skCol = B.&skCol AND A.&extCol = B.&extCol)
           AND NOT &pageNm = 'Brief Custom Details') BY SASSPDS;

EXECUTE (CREATE TABLE Z&udfTable AS (SELECT A.&skCol, A.&pageNm, A.&udfNmCol, A.&udfValCol, B.&extCol, A.&procDttm
                                       FROM X&udfTable A, &udfTable B
                                      WHERE A.&skCol = B.&skCol
                                        AND A.&pageNm = B.&pageNm
                                        AND A.&udfNmCol = B.&udfNmCol
                                        AND A.&udfValCol = B.&udfValCol
                                        AND A.&procDttm = B.&procDttm)) BY SASSPDS;

EXECUTE (INSERT INTO Z&udfTable (&skCol, &pageNm, &udfNmCol, &udfValCol, &extCol, &procDttm)
              SELECT &skCol, &pageNm, &udfNmCol, &udfValCol, &extCol, &procDttm
                FROM BA&udfTable A
               WHERE NOT EXISTS (SELECT &skCol FROM Z&udfTable B WHERE A.&skCol = B.&skCol
                                                                   AND A.&pageNm = B.&pageNm
                                                                   AND A.&udfNmCol = B.&udfNmCol
                                                                   AND A.&udfValCol = B.&udfValCol
                                                                   AND A.&procDttm = B.&procDttm)) BY SASSPDS;

EXECUTE (INSERT INTO Z&udfTable (&skCol, &pageNm, &udfNmCol, &udfValCol, &extCol, &procDttm)
              SELECT &skCol, &pageNm, &udfNmCol, &udfValCol, &extCol, &procDttm
                FROM DA&udfTable A
               WHERE NOT EXISTS (SELECT &skCol FROM Z&udfTable B WHERE A.&skCol = B.&skCol
                                                                   AND A.&pageNm = B.&pageNm
                                                                   AND A.&udfNmCol = B.&udfNmCol
                                                                   AND A.&udfValCol = B.&udfValCol
                                                                   AND A.&procDttm = B.&procDttm)) BY SASSPDS;

EXECUTE (DELETE FROM &udfTable) BY SASSPDS;

EXECUTE (INSERT INTO &udfTable (&skCol, &pageNm, &udfNmCol, &udfValCol, &extCol, &procDttm)
              SELECT &skCol, &pageNm, &udfNmCol, &udfValCol, &extCol, &procDttm FROM Z&udfTable) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/* Cleanup */
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

EXECUTE (DROP TABLE BX&udfTable) BY SASSPDS;
EXECUTE (DROP TABLE DX&udfTable) BY SASSPDS;
EXECUTE (DROP TABLE BA&udfTable) BY SASSPDS;
EXECUTE (DROP TABLE DA&udfTable) BY SASSPDS;
EXECUTE (DROP TABLE X&udfTable) BY SASSPDS;
EXECUTE (DROP TABLE Z&udfTable) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


%mend;


/** UPDATE DUPLICATE EXT COLUMN NAMES **/
%UpdateColumns(CI_CAMP_PAGE_CHAR_UDF, CAMPAIGN_SK, CHAR_UDF_NM, CHAR_UDF_VAL, CHAR_EXT_COLUMN_NM);
%UpdateColumns(CI_CAMP_PAGE_NUM_UDF, CAMPAIGN_SK, NUM_UDF_NM, NUM_UDF_VAL, NUM_EXT_COLUMN_NM);
%UpdateColumns(CI_CAMP_PAGE_DATE_UDF, CAMPAIGN_SK, DATE_UDF_NM, DATE_UDF_VAL, DATE_EXT_COLUMN_NM);


/********************************************************************/
/*                                                                  */
/* Migration is complete on backup CAMPAIGN tables but we need to   */
/* replace some of existing tables and delete others.               */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* EXISTING CHECKLIST TABLES ARE MODIFIED AT THE END OF THIS SCRIPT */
/********************************************************************/


/********************************************************************/
/******************  CI CAMPAIGN GROUP TABLES  **********************/
/********************************************************************/


/** CREATE BACKUP OF MODIFIED TABLES TO DO THE WORK **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Create a backup of existing tables to be migrated (just incase) **/
EXECUTE (CREATE TABLE SV_CI_CAMP_GRP_CHECKLIST_ITEM AS (SELECT * FROM CI_CAMP_GRP_CHECKLIST_ITEM)) BY SASSPDS;
EXECUTE (CREATE TABLE SV_CI_CAMP_GRP_CLIST_ITEM_EXT AS (SELECT * FROM CI_CAMP_GRP_CLIST_ITEM_EXT)) BY SASSPDS;

EXECUTE (CREATE TABLE SV_CI_CAMPAIGN_GROUP_EXT AS (SELECT * FROM CI_CAMPAIGN_GROUP_EXT)) BY SASSPDS;

EXECUTE (CREATE TABLE SV_CI_CAMPAIGN_GROUP_CHAR_UDF AS (SELECT * FROM CI_CAMPAIGN_GROUP_CHAR_UDF)) BY SASSPDS;
EXECUTE (CREATE TABLE SV_CI_CAMPAIGN_GROUP_NUM_UDF AS (SELECT * FROM CI_CAMPAIGN_GROUP_NUM_UDF)) BY SASSPDS;
EXECUTE (CREATE TABLE SV_CI_CAMPAIGN_GROUP_DATE_UDF AS (SELECT * FROM CI_CAMPAIGN_GROUP_DATE_UDF)) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** UPDATE CHECKLIST NAME COLUMN **/
%UpdateBriefItem(CI_CAMP_GRP_CHECKLIST_ITEM);


/** UPDATE DATA IN THE NEW UDF COLUMN **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Update column CHECKLIST_NM values in UDF tables **/
EXECUTE (CREATE TABLE SV_CI_CAMP_GRP_CLIST_CHAR_UDF AS SELECT A.CAMPAIGN_GROUP_SK,
                                                           B.CHECKLIST_NM,
                                                           A.UDF_SEQ_NO,
                                                           A.CHAR_UDF_NM,
                                                           A.CHAR_UDF_VAL,
                                                           A.CHAR_EXT_COLUMN_NM,
                                                           A.PROCESSED_DTTM
                                                      FROM CI_CAMP_GRP_CLIST_CHAR_UDF A, SV_CI_CAMP_GRP_CHECKLIST_ITEM B
                                                     WHERE A.CAMPAIGN_GROUP_SK = B.CAMPAIGN_GROUP_SK AND A.UDF_SEQ_NO = B.UDF_SEQ_NO) BY SASSPDS;

EXECUTE (CREATE TABLE SV_CI_CAMP_GRP_CLIST_NUM_UDF AS SELECT A.CAMPAIGN_GROUP_SK,
                                                          B.CHECKLIST_NM,
                                                          A.UDF_SEQ_NO,
                                                          A.NUM_UDF_NM,
                                                          A.NUM_UDF_VAL,
                                                          A.NUM_EXT_COLUMN_NM,
                                                          A.PROCESSED_DTTM
                                                     FROM CI_CAMP_GRP_CLIST_NUM_UDF A, SV_CI_CAMP_GRP_CHECKLIST_ITEM B
                                                    WHERE A.CAMPAIGN_GROUP_SK = B.CAMPAIGN_GROUP_SK AND A.UDF_SEQ_NO = B.UDF_SEQ_NO) BY SASSPDS;

EXECUTE (CREATE TABLE SV_CI_CAMP_GRP_CLIST_DATE_UDF AS SELECT A.CAMPAIGN_GROUP_SK,
                                                           B.CHECKLIST_NM,
                                                           A.UDF_SEQ_NO,
                                                           A.DATE_UDF_NM,
                                                           A.DATE_UDF_VAL,
                                                           A.DATE_EXT_COLUMN_NM,
                                                           A.PROCESSED_DTTM
                                                      FROM CI_CAMP_GRP_CLIST_DATE_UDF A, SV_CI_CAMP_GRP_CHECKLIST_ITEM B
                                                     WHERE A.CAMPAIGN_GROUP_SK = B.CAMPAIGN_GROUP_SK AND A.UDF_SEQ_NO = B.UDF_SEQ_NO) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** POPULATE BRIEF DATA **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

EXECUTE (CREATE TABLE TM_CI_CAMP_GRP_CHECKLIST_ITEM AS (SELECT * FROM SV_CI_CAMP_GRP_CHECKLIST_ITEM)) BY SASSPDS;

/** Add Brief Page if not already present **/
EXECUTE (INSERT INTO SV_CI_CAMP_GRP_CHECKLIST_ITEM
      (CAMPAIGN_GROUP_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       CHECKLIST_STATUS_FLG)
   SELECT DISTINCT CAMPAIGN_GROUP_SK,
          -1,
          'Brief Custom Details',
          'Y'
   FROM SV_CI_CAMPAIGN_GROUP_EXT A
   WHERE NOT EXISTS (SELECT DISTINCT CAMPAIGN_GROUP_SK FROM TM_CI_CAMP_GRP_CHECKLIST_ITEM B
                     WHERE B.CAMPAIGN_GROUP_SK = A.CAMPAIGN_GROUP_SK
                       AND B.CHECKLIST_NM = 'Brief Custom Details')) BY SASSPDS;

EXECUTE (DELETE FROM TM_CI_CAMP_GRP_CHECKLIST_ITEM) BY SASSPDS;
EXECUTE (INSERT INTO TM_CI_CAMP_GRP_CHECKLIST_ITEM SELECT * FROM SV_CI_CAMP_GRP_CHECKLIST_ITEM) BY SASSPDS;

/** Add empty columns if not already there **/
EXECUTE (INSERT INTO SV_CI_CAMP_GRP_CHECKLIST_ITEM
      (CAMPAIGN_GROUP_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       CHECKLIST_STATUS_FLG)
   SELECT DISTINCT CAMPAIGN_GROUP_SK,
          -1,
          'Brief Custom Details',
          'Y'
   FROM SV_CI_CAMPAIGN_GROUP_CHAR_UDF A
   WHERE NOT EXISTS (SELECT DISTINCT CAMPAIGN_GROUP_SK FROM TM_CI_CAMP_GRP_CHECKLIST_ITEM B
                     WHERE B.CAMPAIGN_GROUP_SK = A.CAMPAIGN_GROUP_SK
                       AND B.UDF_SEQ_NO = -1)) BY SASSPDS;

EXECUTE (DELETE FROM TM_CI_CAMP_GRP_CHECKLIST_ITEM) BY SASSPDS;
EXECUTE (INSERT INTO TM_CI_CAMP_GRP_CHECKLIST_ITEM SELECT * FROM SV_CI_CAMP_GRP_CHECKLIST_ITEM) BY SASSPDS;

EXECUTE (INSERT INTO SV_CI_CAMP_GRP_CHECKLIST_ITEM
      (CAMPAIGN_GROUP_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       CHECKLIST_STATUS_FLG)
   SELECT DISTINCT CAMPAIGN_GROUP_SK,
          -1,
          'Brief Custom Details',
          'Y'
   FROM SV_CI_CAMPAIGN_GROUP_NUM_UDF A
   WHERE NOT EXISTS (SELECT DISTINCT CAMPAIGN_GROUP_SK FROM TM_CI_CAMP_GRP_CHECKLIST_ITEM B
                     WHERE B.CAMPAIGN_GROUP_SK = A.CAMPAIGN_GROUP_SK
                       AND B.UDF_SEQ_NO = -1)) BY SASSPDS;

EXECUTE (DELETE FROM TM_CI_CAMP_GRP_CHECKLIST_ITEM) BY SASSPDS;
EXECUTE (INSERT INTO TM_CI_CAMP_GRP_CHECKLIST_ITEM SELECT * FROM SV_CI_CAMP_GRP_CHECKLIST_ITEM) BY SASSPDS;

EXECUTE (INSERT INTO SV_CI_CAMP_GRP_CHECKLIST_ITEM
      (CAMPAIGN_GROUP_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       CHECKLIST_STATUS_FLG)
   SELECT DISTINCT CAMPAIGN_GROUP_SK,
          -1,
          'Brief Custom Details',
          'Y'
   FROM SV_CI_CAMPAIGN_GROUP_DATE_UDF A
   WHERE NOT EXISTS (SELECT CAMPAIGN_GROUP_SK FROM TM_CI_CAMP_GRP_CHECKLIST_ITEM B
                     WHERE B.CAMPAIGN_GROUP_SK = A.CAMPAIGN_GROUP_SK
                       AND B.UDF_SEQ_NO = -1)) BY SASSPDS;

EXECUTE (DELETE FROM TM_CI_CAMP_GRP_CHECKLIST_ITEM) BY SASSPDS;
EXECUTE (INSERT INTO TM_CI_CAMP_GRP_CHECKLIST_ITEM SELECT * FROM SV_CI_CAMP_GRP_CHECKLIST_ITEM) BY SASSPDS;

/** Update processed date value (not necessary - values should exist) **/
/*UPDATE &LIB..SV_CI_CAMP_GRP_CHECKLIST_ITEM A
   SET PROCESSED_DTTM = (SELECT PROCESSED_DTTM FROM &LIB..CI_CAMPAIGN_GROUP B
                         WHERE (A.CAMPAIGN_GROUP_SK = B.CAMPAIGN_GROUP_SK AND B.PROCESSED_DTTM IS NOT NULL))
   WHERE A.CAMPAIGN_GROUP_SK IN (SELECT CAMPAIGN_GROUP_SK FROM &LIB..CI_CAMPAIGN_GROUP) AND A.UDF_SEQ_NO = -1;*/

/** Copy Brief UDFs **/
EXECUTE (INSERT INTO SV_CI_CAMP_GRP_CLIST_CHAR_UDF
      (CAMPAIGN_GROUP_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       CHAR_UDF_NM,
       CHAR_UDF_VAL,
       CHAR_EXT_COLUMN_NM,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_GROUP_SK,
          -1,
          'Brief Custom Details',
          CHAR_UDF_NM,
          CHAR_UDF_VAL,
          CHAR_EXT_COLUMN_NM,
          PROCESSED_DTTM
   FROM SV_CI_CAMPAIGN_GROUP_CHAR_UDF) BY SASSPDS;

EXECUTE (INSERT INTO SV_CI_CAMP_GRP_CLIST_NUM_UDF
      (CAMPAIGN_GROUP_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       NUM_UDF_NM,
       NUM_UDF_VAL,
       NUM_EXT_COLUMN_NM,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_GROUP_SK,
          -1,
          'Brief Custom Details',
          NUM_UDF_NM,
          NUM_UDF_VAL,
          NUM_EXT_COLUMN_NM,
          PROCESSED_DTTM
   FROM SV_CI_CAMPAIGN_GROUP_NUM_UDF) BY SASSPDS;

EXECUTE (INSERT INTO SV_CI_CAMP_GRP_CLIST_DATE_UDF
      (CAMPAIGN_GROUP_SK,
       UDF_SEQ_NO,
       CHECKLIST_NM,
       DATE_UDF_NM,
       DATE_UDF_VAL,
       DATE_EXT_COLUMN_NM,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_GROUP_SK,
          -1,
          'Brief Custom Details',
          DATE_UDF_NM,
          DATE_UDF_VAL,
          DATE_EXT_COLUMN_NM,
          PROCESSED_DTTM
   FROM SV_CI_CAMPAIGN_GROUP_DATE_UDF) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** REMOVE NULL OR DUPLICATES ON PK COLUMNS **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Remove rows with null on page name (PK) **/
EXECUTE (DELETE FROM SV_CI_CAMP_GRP_CLIST_CHAR_UDF WHERE CHECKLIST_NM IS NULL) BY SASSPDS;
EXECUTE (DELETE FROM SV_CI_CAMP_GRP_CLIST_NUM_UDF WHERE CHECKLIST_NM IS NULL) BY SASSPDS;
EXECUTE (DELETE FROM SV_CI_CAMP_GRP_CLIST_DATE_UDF WHERE CHECKLIST_NM IS NULL) BY SASSPDS;
EXECUTE (DELETE FROM SV_CI_CAMP_GRP_CHECKLIST_ITEM WHERE CHECKLIST_NM IS NULL) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/*** Run macro to add all columns to EXT ***/
%extunion(&LIB..SV_CI_CAMP_GRP_CLIST_ITEM_EXT, &LIB..SV_CI_CAMPAIGN_GROUP_EXT, &LIB..SMIG_CI_CAMPAIGN_GROUP_EXT, CAMPAIGN_GROUP_SK);

PROC APPEND BASE=&LIB..SMIG_CI_CAMPAIGN_GROUP_EXT DATA=&LIB..SV_CI_CAMPAIGN_GROUP_EXT FORCE; run;

/*** Run macro to migrate campaign group EXT ***/
%extmigrate(&LIB..SV_CI_CAMP_GRP_CLIST_ITEM_EXT, &LIB..SMIG_CI_CAMPAIGN_GROUP_EXT, &LIB..MIG_CI_CAMPAIGN_GROUP_EXT, CAMPAIGN_GROUP_SK);


/** CREATE NEW TABLES **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Create Tables **/
EXECUTE (CREATE TABLE CI_CAMP_GRP_PAGE
(
	CAMPAIGN_GROUP_SK    NUMERIC(10)  label='Campaign Group Surrogate Key',
	PAGE_NM              VARCHAR(60) label='PAGE Name',
	PAGE_STATUS_FLG CHARACTER(1) label='PAGE Status Flag',
	PROCESSED_DTTM       DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Processed Datetime'
)) BY SASSPDS;

EXECUTE (CREATE TABLE CI_CAMP_GRP_PAGE_CHAR_UDF
(
	CAMPAIGN_GROUP_SK    NUMERIC(10)  label='Campaign Group Surrogate Key',
	PAGE_NM              VARCHAR(60) label='Page Name',
	CHAR_UDF_NM          VARCHAR(60) label='Char Udf Name',
	CHAR_UDF_VAL         VARCHAR(500) label='Char Udf Val',
	CHAR_EXT_COLUMN_NM   VARCHAR(30) label='Char Ext Column Name',
	PROCESSED_DTTM       DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Processed Datetime'
)) BY SASSPDS;

EXECUTE (CREATE TABLE CI_CAMP_GRP_PAGE_NUM_UDF
(
	CAMPAIGN_GROUP_SK    NUMERIC(10)  label='Campaign Group Surrogate Key',
	PAGE_NM              VARCHAR(60) label='Page Name',
	NUM_UDF_NM           VARCHAR(60) label='Num Udf Name',
	NUM_UDF_VAL          NUMERIC(14,2) label='Num Udf Val',
	NUM_EXT_COLUMN_NM    VARCHAR(30) label='Num Ext Column Name',
	PROCESSED_DTTM       DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Processed Datetime'
)) BY SASSPDS;

EXECUTE (CREATE TABLE CI_CAMP_GRP_PAGE_DATE_UDF
(
	CAMPAIGN_GROUP_SK    NUMERIC(10)  label='Campaign Group Surrogate Key',
	PAGE_NM              VARCHAR(60) label='Page Name',
	DATE_UDF_NM          VARCHAR(60) label='Date Udf Name',
	DATE_UDF_VAL         DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Date Udf Val',
	DATE_EXT_COLUMN_NM   VARCHAR(30) label='Date Ext Column Name',
	PROCESSED_DTTM       DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Processed Datetime'
)) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** ADD CONSTRAINTS TO NEW TABLES **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Create constraints on new tables **/
EXECUTE (CREATE UNIQUE INDEX PRIMARY_KEY ON CI_CAMP_GRP_PAGE (CAMPAIGN_GROUP_SK, PAGE_NM)) BY SASSPDS;

EXECUTE (CREATE UNIQUE INDEX PRIMARY_KEY ON CI_CAMP_GRP_PAGE_CHAR_UDF (CAMPAIGN_GROUP_SK, PAGE_NM, CHAR_UDF_NM)) BY SASSPDS;

EXECUTE (CREATE UNIQUE INDEX PRIMARY_KEY ON CI_CAMP_GRP_PAGE_NUM_UDF (CAMPAIGN_GROUP_SK, PAGE_NM, NUM_UDF_NM)) BY SASSPDS;

EXECUTE (CREATE UNIQUE INDEX PRIMARY_KEY ON CI_CAMP_GRP_PAGE_DATE_UDF (CAMPAIGN_GROUP_SK, PAGE_NM, DATE_UDF_NM)) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** POPULATE NEW TABLES FROM EXISTING DATA **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Add Existing data to new tables **/
EXECUTE (INSERT INTO CI_CAMP_GRP_PAGE
      (CAMPAIGN_GROUP_SK,
       PAGE_NM,
       PAGE_STATUS_FLG,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_GROUP_SK,
          CHECKLIST_NM,
          CHECKLIST_STATUS_FLG,
          PROCESSED_DTTM
   FROM SV_CI_CAMP_GRP_CHECKLIST_ITEM) BY SASSPDS;

EXECUTE (INSERT INTO CI_CAMP_GRP_PAGE_CHAR_UDF
      (CAMPAIGN_GROUP_SK,
       PAGE_NM,
       CHAR_UDF_NM,
       CHAR_UDF_VAL,
       CHAR_EXT_COLUMN_NM,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_GROUP_SK,
          CHECKLIST_NM,
          CHAR_UDF_NM,
          CHAR_UDF_VAL,
          CHAR_EXT_COLUMN_NM,
          PROCESSED_DTTM
   FROM SV_CI_CAMP_GRP_CLIST_CHAR_UDF) BY SASSPDS;

EXECUTE (INSERT INTO CI_CAMP_GRP_PAGE_NUM_UDF
      (CAMPAIGN_GROUP_SK,
       PAGE_NM,
       NUM_UDF_NM,
       NUM_UDF_VAL,
       NUM_EXT_COLUMN_NM,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_GROUP_SK,
          CHECKLIST_NM,
          NUM_UDF_NM,
          NUM_UDF_VAL,
          NUM_EXT_COLUMN_NM,
          PROCESSED_DTTM
   FROM SV_CI_CAMP_GRP_CLIST_NUM_UDF) BY SASSPDS;

EXECUTE (INSERT INTO CI_CAMP_GRP_PAGE_DATE_UDF
      (CAMPAIGN_GROUP_SK,
       PAGE_NM,
       DATE_UDF_NM,
       DATE_UDF_VAL,
       DATE_EXT_COLUMN_NM,
       PROCESSED_DTTM)
   SELECT CAMPAIGN_GROUP_SK,
          CHECKLIST_NM,
          DATE_UDF_NM,
          DATE_UDF_VAL,
          DATE_EXT_COLUMN_NM,
          PROCESSED_DTTM
   FROM SV_CI_CAMP_GRP_CLIST_DATE_UDF) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** UPDATE DUPLICATE EXT COLUMN NAMES **/
%UpdateColumns(CI_CAMP_GRP_PAGE_CHAR_UDF, CAMPAIGN_GROUP_SK, CHAR_UDF_NM, CHAR_UDF_VAL, CHAR_EXT_COLUMN_NM);
%UpdateColumns(CI_CAMP_GRP_PAGE_NUM_UDF, CAMPAIGN_GROUP_SK, NUM_UDF_NM, NUM_UDF_VAL, NUM_EXT_COLUMN_NM);
%UpdateColumns(CI_CAMP_GRP_PAGE_DATE_UDF, CAMPAIGN_GROUP_SK, DATE_UDF_NM, DATE_UDF_VAL, DATE_EXT_COLUMN_NM);


/********************************************************************/
/*                                                                  */
/* Migration is complete on backup CAMPAIGN GROUP tables but we     */
/* need to replace some of existing tables and delete others.       */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* EXISTING CHECKLIST TABLES ARE MODIFIED AT THE END OF THIS SCRIPT */
/********************************************************************/

/********************************************************************/
/********************************************************************/


/********************************************************************/
/* Section below modifies existing tables and removes orphaned and  */
/* other temporary tables that are no longer needed.                */
/********************************************************************/

/********************************************************************/
/*********************  CI CAMPAIGN TABLES  *************************/
/********************************************************************/



/* Get EXT columns order */
%macro GetEXTColumns(TableName, pkCol);

proc contents noprint data=&LIB..&TableName out=Z&TableName; run;

%let keepVars=.;
%let keepTypes=.;
%let keepVarNum=.;
%let maxNumber=.;

data V&TableName;
   format nu : 10.;
   length nm $ 256;
   length tm $ 256;
   length v $ 32767;
   length t $ 32767;
   length u $ 32767;
   length s $ 32767;
   format w : 10.;
   retain v t u w;
   keep nu nm tp;
   set Z&TableName end=done;
   v=trim(v)||" "||upcase(trim(name));
   u=trim(u)||" "||trim(left(varnum));
   nu=trim(left(varnum))+0;
   nm=upcase(trim(name));
   if missing(w) or w lt trim(left(varnum)) then do;
      w=trim(left(varnum));
   end;
   if missing(format) and type eq 1 then do;
      if not missing(formatd) and formatd gt 0 and not missing(formatl) and formatl gt 0 then do;
         t=trim(t)||" NUMERIC("||trim(left(formatl))||","||trim(left(formatd))||")";
         tp="NUMERIC("||trim(left(formatl))||","||trim(left(formatd))||")";
      end;
      else do;
         if not missing(formatl) and formatl gt 0 then do;
            if &pkCol eq upcase(trim(name)) then do;
               t=trim(t)||" NUMERIC(10)";
               tp="NUMERIC(10)";
            end;
            else do;
               t=trim(t)||" NUMERIC("||trim(left(formatl))||")";
               tp="NUMERIC("||trim(left(formatl))||")";
            end;
         end;
         else do;
            t=trim(t)||" NUMERIC";
            tp="NUMERIC";
         end;
      end;
   end;
   else do;
      if not missing(format) and upcase(format) eq 'DATETIME' then do;
         if not missing(formatl) and formatl eq 20 then do;
            t=trim(t)||" DATE";
            tp="DATE";
         end;
         else do;
            t=trim(t)||" TIMESTAMP";
            tp="TIMESTAMP";;
         end;
      end;
      else do;
         if not missing(format) and upcase(format) eq 'DATE' then do;
            t=trim(t)||" DATE";
            tp="DATE";
         end;
         else do;
            if type eq 1 then do;
               if not missing(formatd) and formatd gt 0 and not missing(formatl) and formatl gt 0 then do;
                  t=trim(t)||" NUMERIC("||trim(left(formatl))||","||trim(left(formatd))||")";
                  tp="NUMERIC("||trim(left(formatl))||","||trim(left(formatd))||")";
               end;
               else do;
                  if not missing(formatl) and formatl gt 0 then do;
                     if &pkCol eq upcase(trim(name)) then do;
                        t=trim(t)||" NUMERIC(10)";
                        tp="NUMERIC(10)";
                     end;
                     else do;
                        t=trim(t)||" NUMERIC("||trim(left(formatl))||")";
                        tp="NUMERIC("||trim(left(formatl))||")";
                     end;
                  end;
                  else do;
                     t=trim(t)||" NUMERIC";
                     tp="NUMERIC";
                  end;
               end;
            end;
            else do;
               if not missing(formatl) and formatl gt 0 then do;
                  t=trim(t)||" VARCHAR("||trim(left(formatl))||")";
                  tp="VARCHAR("||trim(left(formatl))||")";
               end;
                else do;
                  t=trim(t)||" VARCHAR";
                  tp="VARCHAR";
               end;
            end;
         end;
      end;
   end;
   if done then do;
      call symput('keepVars',trim(v));
      call symput('keepTypes',trim(t));
      call symput('keepVarNum',trim(u));
      call symput('maxNumber',trim(w));
   end;
run;

%put Columns=%trim(&keepVars);
%put Types=%trim(&keepTypes);
%put Numbers=%trim(&keepVarNum);
%put Max=%trim(%left(&maxNumber));

%mend;


/* Recreate EXT table with same column order as original */
%macro CreateEXTTable(TTableName, STableName, pk_column);

%put &STableName &TTableName;

%let keepVars=.;
%let keepTypes=.;
%let keepVarNum=.;
%let maxNumber=.;

/* Get orignal EXT columns */
%GetEXTColumns(&TTableName, "&pk_column.");
%let origCols=%trim(&keepVars);
%let origTypes=%trim(&keepTypes);
%let origVarNum=%trim(&keepVarNum);
%let origMaxNumber=%trim(%left(&maxNumber));

/* Get latest EXT columns */
%GetEXTColumns(&STableName, "&pk_column.");
%let srcCols=%trim(&keepVars);
%let srcTypes=%trim(&keepTypes);
%let srcVarNum=%trim(&keepVarNum);
%let srcMaxNumber=%trim(&maxNumber);


PROC SQL NOERRORSTOP;

CREATE TABLE NV&STableName AS (SELECT nu, nm, tp FROM V&STableName WHERE NM NOT IN (SELECT NM FROM V&TTableName));

QUIT;


%let addedCol=.;
%let addedType=.;
%let addedNum=.;
data UNV&STableName;
   length c $ 256;
   length t $ 256;
   length n $ 256;
   format b : 10.;
   format nub : 10.;
   retain t c n nub;
   keep nub nm tp;
   set NV&STableName end=alldone;
   if _n_=1 then do;
      nub=left(&origMaxNumber+1)+0;
   end;
   else do;
      nub=left(nub+1)+0;
   end;
   t=trim(t)||' '||trim(tp);
   c=trim(c)||' '||trim(nm);
   n=trim(n)||' '||trim(left(nub));
   if alldone then do;
      call symput('addedCol',trim(c));
      call symput('addedType',trim(t));
      call symput('addedNum',trim(n));
   end;
run;
%put addedCol=%trim(&addedCol);
%put addedType=%trim(&addedType);
%put addedNum=%trim(&addedNum);


PROC SQL NOERRORSTOP;

CREATE TABLE AV&STableName
(
    COLNUMBER NUMERIC(10) NOT NULL,
    COLNAME VARCHAR(256),
    COLTYPE VARCHAR(256)
);

INSERT INTO AV&STableName (COLNUMBER, COLNAME, COLTYPE) SELECT NU, NM, TP FROM V&TTableName;
INSERT INTO AV&STableName (COLNUMBER, COLNAME, COLTYPE) SELECT NUB, NM, TP FROM UNV&STableName;
CREATE TABLE AVS&STableName AS SELECT COLNUMBER, COLNAME, COLTYPE FROM AV&STableName ORDER BY COLNUMBER;

QUIT;


%let fv=1;
%let lv=1;
data _null_;
   retain fv lv;
   set AVS&STableName end=sdone;
   if _n_=1 then do;
      fv=left(COLNUMBER);
      lv=left(COLNUMBER);
   end;
   else do;
      if fv gt COLNUMBER then do;
         fv=left(COLNUMBER);
      end;
      if lv lt COLNUMBER then do;
         lv=left(COLNUMBER);
      end;
   end;
   call symput('cn'||trim(left(COLNUMBER)),trim(COLNAME));
   call symput('ct'||trim(left(COLNUMBER)),trim(COLTYPE));
   if sdone then do;
      call symput('fv',trim(fv));
      call symput('lv',trim(lv));
   end;
run;

%put fv=&fv;
%put lv=&lv;

%do i = &fv %to &lv;
   %put colum[&i]=%str(&&cn&i) %str(&&ct&i);
%end;


/** REPLACE EXT TABLE **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

EXECUTE (DROP TABLE &TTableName) BY SASSPDS;

EXECUTE (CREATE TABLE &TTableName AS
(
SELECT
    %do i = &fv %to &lv;
       %str(&&cn&i)
       %if &i lt &lv %then %do;
          %str(, )
       %end;
    %end;
FROM &STableName)) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/* Cleanup */
PROC SQL NOERRORSTOP;

DROP TABLE AVS&STableName;
DROP TABLE AV&STableName;
DROP TABLE NV&STableName;
DROP TABLE UNV&STableName;
DROP TABLE V&TTableName;
DROP TABLE V&STableName;
DROP TABLE Z&TTableName;
DROP TABLE Z&STableName;

QUIT;


%mend;


/* Replace campaign EXT */
%CreateEXTTable(CI_CAMPAIGN_EXT, MIG_CI_CAMPAIGN_EXT, CAMPAIGN_SK);


/** REPLACE CAMPAIGN EXT TABLE **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Add constraints to new table **/
EXECUTE (CREATE UNIQUE INDEX CAMPAIGN_SK ON CI_CAMPAIGN_EXT (CAMPAIGN_SK)) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** DELETE OLD TABLES (REPLACED BY VIEWS) **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Drop old (replaced by views) tables **/
EXECUTE (DROP TABLE CI_CAMPAIGN_CHAR_UDF) BY SASSPDS;
EXECUTE (DROP TABLE CI_CAMPAIGN_NUM_UDF) BY SASSPDS;
EXECUTE (DROP TABLE CI_CAMPAIGN_DATE_UDF) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** CREATE UDF VIEWS **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Create Views **/
EXECUTE (CREATE VIEW CI_CAMPAIGN_CHAR_UDF AS
    SELECT CAMPAIGN_SK ,
           CHAR_UDF_NM ,
           CHAR_UDF_VAL,
           CHAR_EXT_COLUMN_NM ,
           PROCESSED_DTTM
    FROM CI_CAMP_PAGE_CHAR_UDF WHERE PAGE_NM IN  ('Brief Custom Details','BRIEF CUSTOM DETAILS')) BY SASSPDS;

EXECUTE (CREATE VIEW CI_CAMPAIGN_NUM_UDF AS
    SELECT CAMPAIGN_SK ,
           NUM_UDF_NM ,
           NUM_UDF_VAL ,
           NUM_EXT_COLUMN_NM  ,
           PROCESSED_DTTM
    FROM CI_CAMP_PAGE_NUM_UDF WHERE PAGE_NM IN  ('Brief Custom Details','BRIEF CUSTOM DETAILS')) BY SASSPDS;

EXECUTE (CREATE VIEW CI_CAMPAIGN_DATE_UDF AS
    SELECT CAMPAIGN_SK ,
           DATE_UDF_NM  ,
           DATE_UDF_VAL ,
           DATE_EXT_COLUMN_NM  ,
           PROCESSED_DTTM
    FROM CI_CAMP_PAGE_DATE_UDF WHERE PAGE_NM IN  ('Brief Custom Details','BRIEF CUSTOM DETAILS')) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** DELETE ORPHANED AND MIGRATION BACKUP TABLES **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Drop unused tables **/
EXECUTE (DROP TABLE CI_CAMP_CHECKLIST_ITEM) BY SASSPDS;
EXECUTE (DROP TABLE CI_CAMP_CHECKLIST_ITEM_EXT) BY SASSPDS;
EXECUTE (DROP TABLE CI_CAMP_CHECKLIST_CHAR_UDF) BY SASSPDS;
EXECUTE (DROP TABLE CI_CAMP_CHECKLIST_NUM_UDF) BY SASSPDS;
EXECUTE (DROP TABLE CI_CAMP_CHECKLIST_DATE_UDF) BY SASSPDS;

/** Drop migration table **/
EXECUTE (DROP TABLE MIG_CI_CAMPAIGN_EXT) BY SASSPDS;
EXECUTE (DROP TABLE SMIG_CI_CAMPAIGN_EXT) BY SASSPDS;

/** Drop backup (saved) tables **/
EXECUTE (DROP TABLE SV_CI_CAMP_CHECKLIST_ITEM) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMP_CHECKLIST_ITEM_EXT) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMP_CHECKLIST_CHAR_UDF) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMP_CHECKLIST_NUM_UDF) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMP_CHECKLIST_DATE_UDF) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMPAIGN_EXT) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMPAIGN_CHAR_UDF) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMPAIGN_NUM_UDF) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMPAIGN_DATE_UDF) BY SASSPDS;

EXECUTE (DROP TABLE TM_CI_CAMP_CHECKLIST_ITEM) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;

/********************************************************************/


/********************************************************************/
/******************  CI CAMPAIGN GROUP TABLES  **********************/
/********************************************************************/


/* Replace campaign group EXT */
%CreateEXTTable(CI_CAMPAIGN_GROUP_EXT, MIG_CI_CAMPAIGN_GROUP_EXT, CAMPAIGN_GROUP_SK);

/** REPLACE CAMPAIGN EXT TABLE **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Add constraints to new table **/
EXECUTE (CREATE UNIQUE INDEX CAMPAIGN_GROUP_SK ON CI_CAMPAIGN_GROUP_EXT (CAMPAIGN_GROUP_SK)) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** DELETE OLD TABLES (REPLACED BY VIEWS) **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Drop old (replaced by views) tables **/
EXECUTE (DROP TABLE CI_CAMPAIGN_GROUP_CHAR_UDF) BY SASSPDS;
EXECUTE (DROP TABLE CI_CAMPAIGN_GROUP_NUM_UDF) BY SASSPDS;
EXECUTE (DROP TABLE CI_CAMPAIGN_GROUP_DATE_UDF) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** CREATE UDF VIEWS **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Create Views **/
EXECUTE (CREATE VIEW CI_CAMPAIGN_GROUP_CHAR_UDF AS
    SELECT CAMPAIGN_GROUP_SK  ,
           CHAR_UDF_NM  ,
           CHAR_UDF_VAL  ,
           CHAR_EXT_COLUMN_NM ,
           PROCESSED_DTTM
    FROM CI_CAMP_GRP_PAGE_CHAR_UDF WHERE PAGE_NM IN  ('Brief Custom Details','BRIEF CUSTOM DETAILS')) BY SASSPDS;

EXECUTE (CREATE VIEW CI_CAMPAIGN_GROUP_NUM_UDF AS
    SELECT CAMPAIGN_GROUP_SK ,
           NUM_UDF_NM   ,
           NUM_UDF_VAL  ,
           NUM_EXT_COLUMN_NM ,
           PROCESSED_DTTM
    FROM CI_CAMP_GRP_PAGE_NUM_UDF WHERE PAGE_NM IN  ('Brief Custom Details','BRIEF CUSTOM DETAILS')) BY SASSPDS;

EXECUTE (CREATE VIEW CI_CAMPAIGN_GROUP_DATE_UDF AS
    SELECT CAMPAIGN_GROUP_SK ,
           DATE_UDF_NM  ,
           DATE_UDF_VAL  ,
           DATE_EXT_COLUMN_NM  ,
           PROCESSED_DTTM
    FROM CI_CAMP_GRP_PAGE_DATE_UDF WHERE PAGE_NM IN  ('Brief Custom Details','BRIEF CUSTOM DETAILS')) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/** DELETE ORPHANED AND MIGRATION BACKUP TABLES **/
PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

/** Drop unused tables **/
EXECUTE (DROP TABLE CI_CAMP_GRP_CHECKLIST_ITEM) BY SASSPDS;
EXECUTE (DROP TABLE CI_CAMP_GRP_CLIST_ITEM_EXT) BY SASSPDS;
EXECUTE (DROP TABLE CI_CAMP_GRP_CLIST_CHAR_UDF) BY SASSPDS;
EXECUTE (DROP TABLE CI_CAMP_GRP_CLIST_NUM_UDF) BY SASSPDS;
EXECUTE (DROP TABLE CI_CAMP_GRP_CLIST_DATE_UDF) BY SASSPDS;

/** Drop migration table **/
EXECUTE (DROP TABLE MIG_CI_CAMPAIGN_GROUP_EXT) BY SASSPDS;
EXECUTE (DROP TABLE SMIG_CI_CAMPAIGN_GROUP_EXT) BY SASSPDS;

/** Drop backup (saved) tables **/
EXECUTE (DROP TABLE SV_CI_CAMP_GRP_CHECKLIST_ITEM) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMP_GRP_CLIST_ITEM_EXT) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMP_GRP_CLIST_CHAR_UDF) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMP_GRP_CLIST_NUM_UDF) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMP_GRP_CLIST_DATE_UDF) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMPAIGN_GROUP_EXT) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMPAIGN_GROUP_CHAR_UDF) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMPAIGN_GROUP_NUM_UDF) BY SASSPDS;
EXECUTE (DROP TABLE SV_CI_CAMPAIGN_GROUP_DATE_UDF) BY SASSPDS;

EXECUTE (DROP TABLE TM_CI_CAMP_GRP_CHECKLIST_ITEM) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;


/********************************************************************/


/********************************************************************/
/******************  REMOVE WORK SPACE TABLES  **********************/
/********************************************************************/

/** DELETE EXT MERGING WORK TABLES **/
PROC SQL NOERRORSTOP;

DROP TABLE WORK.CONT_EXPANDED;
DROP TABLE WORK.CONT_INTERSECTION;
DROP TABLE WORK.CONT_TARGET;
DROP TABLE WORK.CONT_UNION;
DROP TABLE WORK.DESTINATION_TABLE;
DROP TABLE WORK.OUTPUT_TABLE;
DROP TABLE WORK.SORTED_ITEM_EXT;
DROP TABLE WORK.UNION_ALL;
DROP TABLE WORK.UNION_TABLE;
DROP TABLE WORK.CI_CAMP_CHECKLIST_ITEM;
DROP TABLE WORK.CI_CAMP_GRP_CHECKLIST_ITEM;

QUIT;

/*===================================================================*/
/*                      END CHECKLIST / PAGE                         */
/*===================================================================*/

/***********************************************************/
/***        UPDATE CODE TABLES WITH NEW CODES            ***/
/***********************************************************/

/* Populate CI_CAMPAIGN_STATUS Table (ci) */
%cildcd(&lib..CI_CAMPAIGN_STATUS, SASHELP.ci, CAMPAIGN_STATUS_CD, CAMPAIGN_STATUS_DESC);

/* Populate CI_CHANNEL Table (ci2, ci3) */
%cildcd2(&lib..CI_CHANNEL,SASHELP.ci3,SASHELP.ci2,CHANNEL_CD,CHANNEL_NM,CHANNEL_DESC);

/* Populate CI_COMMUNICATION_STATUS Table (ci4) */
%cildcd(&lib..CI_COMMUNICATION_STATUS, SASHELP.ci4, COMMUNICATION_STATUS_CD, COMMUNICATION_STATUS_DESC);

/* Populate CI_CONTACT_HISTORY_STATUS Table (ci5) */
%cildcd(&lib..CI_CONTACT_HISTORY_STATUS, SASHELP.ci5, CONTACT_HISTORY_STATUS_CD, CONTACT_HISTORY_STATUS_DESC);

/* Populate CI_CONTROL_GROUP_TYPE Table (ci6) */
%cildcd(&lib..CI_CONTROL_GROUP_TYPE, SASHELP.ci6, CONTROL_GROUP_TYPE_CD, CONTROL_GROUP_TYPE_DESC);

/***********************************************************/
/*** DROP COLUMNS FROM EXISTING TABLES FOR 6.1 MIGRATION ***/
/***********************************************************/

PROC SQL NOERRORSTOP;

CONNECT TO SASSPDS (DBQ=&DOMAIN SERVER=&SERVER USER=&USER PASSWD=&PASSWD);

EXECUTE (ALTER TABLE CI_CAMPAIGN_GROUP DROP COLUMN CAMPAIGN_GROUP_OCCURRENCE_NO) BY SASSPDS;

EXECUTE (ALTER TABLE CI_MO_OPTIMIZATION_SCENARIO DROP COLUMN MO_OPT_SCENARIO_PROJECT_NM) BY SASSPDS;

EXECUTE (ALTER TABLE CI_MO_OPTIMIZATION_SCENARIO DROP COLUMN MO_OPT_SCENARIO_PROJECT_DESC) BY SASSPDS;

EXECUTE (ALTER TABLE CI_COMMUNICATION DROP COLUMN ACTUAL_RESPONSE_RT) BY SASSPDS;

DISCONNECT FROM SASSPDS;

QUIT;

/***************END OF SCRIPT ********************/



